precision mediump float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

const lowp int GAUSSIAN_SAMPLES = 9;

vec2 blurCoordinates[GAUSSIAN_SAMPLES];
uniform float texelWidthOffset;
uniform float texelHeightOffset;
uniform bool needScale;//是否需要缩小

//获取模糊后的颜色
vec4 getBlurColor(vec2 newTextureCoordinate){
    lowp vec3 sum = vec3(0.0);
    // Calculate the positions for the blur
    int multiplier = 0;
    vec2 blurStep;
    vec2 singleStepOffset = vec2(texelHeightOffset, texelWidthOffset);

    for (int i = 0; i < GAUSSIAN_SAMPLES; i++)
    {
        multiplier = (i - ((GAUSSIAN_SAMPLES - 1) / 2));
        // Blur in x (horizontal)
        blurStep = float(multiplier) * singleStepOffset;
        blurCoordinates[i] = newTextureCoordinate.xy + blurStep;
    }

    lowp vec4 fragColor=texture2D(inputImageTexture, newTextureCoordinate);

    sum += texture2D(inputImageTexture, blurCoordinates[0]).rgb * 0.05;
    sum += texture2D(inputImageTexture, blurCoordinates[1]).rgb * 0.09;
    sum += texture2D(inputImageTexture, blurCoordinates[2]).rgb * 0.12;
    sum += texture2D(inputImageTexture, blurCoordinates[3]).rgb * 0.15;
    sum += texture2D(inputImageTexture, blurCoordinates[4]).rgb * 0.18;
    sum += texture2D(inputImageTexture, blurCoordinates[5]).rgb * 0.15;
    sum += texture2D(inputImageTexture, blurCoordinates[6]).rgb * 0.12;
    sum += texture2D(inputImageTexture, blurCoordinates[7]).rgb * 0.09;
    sum += texture2D(inputImageTexture, blurCoordinates[8]).rgb * 0.05;
    return vec4(sum, fragColor.a);
}

mat2 scale(vec2 _scale){
    return mat2(_scale.x, 0.0, 0.0, _scale.y);
}

void main(){
    vec2 newTextureCoordinate = textureCoordinate;
    if (texelWidthOffset == 0.0 && texelHeightOffset == 0.0){
        if (needScale){
            newTextureCoordinate -= vec2(0.5);
            newTextureCoordinate = scale(vec2(1.3)) * newTextureCoordinate;
            newTextureCoordinate += vec2(0.5);
        }
        if (newTextureCoordinate.x < 0.0 || newTextureCoordinate.x > 1.0 || newTextureCoordinate.y < 0.0 || newTextureCoordinate.y > 1.0){
            //            if (newTextureCoordinate.x < 0.0){
            //                //                float color = 1.0 - (-newTextureCoordinate.x / 0.2);
            //                float color = -newTextureCoordinate.x * 8.0 / 2.0;
            //                gl_FragColor = vec4(color, color, color, 1.0);
            //
            //            } else if (newTextureCoordinate.x > 1.0){
            //                gl_FragColor = vec4(0.0, 1.0, 0.0, 1.0);
            //
            //            } else if (newTextureCoordinate.y > 1.0){
            //                gl_FragColor = vec4(0.0, 0.0, 1.0, 1.0);
            //
            //            } else {
            //                gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
            //            }
            //            gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
            gl_FragColor = vec4(0.);
        } else {
            gl_FragColor =texture2D(inputImageTexture, newTextureCoordinate);
        }

    } else {
                gl_FragColor = getBlurColor(newTextureCoordinate);
//        gl_FragColor = vec4(1.0, 1.0, 1.0, 1.0);
    }

}