precision mediump float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float textureStepWidth;
uniform float textureStepHeight;
uniform float progress;
const float PI = 3.1415926;

vec2 leftTextureCoordinate;
vec2 rightTextureCoordinate;
vec2 topTextureCoordinate;
vec2 topLeftTextureCoordinate;
vec2 topRightTextureCoordinate;
vec2 bottomTextureCoordinate;
vec2 bottomLeftTextureCoordinate;
vec2 bottomRightTextureCoordinate;
const float threshold = 0.2;
const float quantizationLevels = 10.0;
void main(){
    float amplitude = progress;
    vec2 widthStep = vec2(textureStepWidth, 0.0);//前进的步长
    vec2 heightStep = vec2(0.0, textureStepHeight);//前进的步长
    vec2 widthHeightStep = vec2(textureStepWidth, textureStepHeight);
    vec2 widthNegativeHeightStep = vec2(textureStepWidth, -textureStepHeight);
    //取相邻坐标点
    leftTextureCoordinate = textureCoordinate.xy - widthStep;
    rightTextureCoordinate = textureCoordinate.xy + widthStep;
    topTextureCoordinate = textureCoordinate.xy - heightStep;
    topLeftTextureCoordinate = textureCoordinate.xy - widthHeightStep;
    topRightTextureCoordinate = textureCoordinate.xy + widthNegativeHeightStep;
    bottomTextureCoordinate = textureCoordinate.xy + heightStep;
    bottomLeftTextureCoordinate = textureCoordinate.xy - widthNegativeHeightStep;
    bottomRightTextureCoordinate = textureCoordinate.xy + widthHeightStep;
    vec4 finalColor = texture2D(inputImageTexture, textureCoordinate);
    vec4 textureColor = finalColor;
    vec3 sum = vec3(0.0);
    //计算高斯模糊
    sum += textureColor.rgb * 0.147761;
    sum += texture2D(inputImageTexture, bottomLeftTextureCoordinate).rgb * 0.0947416;
    sum += texture2D(inputImageTexture, bottomRightTextureCoordinate).rgb * 0.0947416;
    sum += texture2D(inputImageTexture, topLeftTextureCoordinate).rgb * 0.0947416;
    sum += texture2D(inputImageTexture, topRightTextureCoordinate).rgb * 0.0947416;
    sum += texture2D(inputImageTexture, topTextureCoordinate).rgb * 0.118318;
    sum += texture2D(inputImageTexture, bottomTextureCoordinate).rgb * 0.118318;
    sum += texture2D(inputImageTexture, leftTextureCoordinate).rgb * 0.118318;
    sum += texture2D(inputImageTexture, rightTextureCoordinate).rgb * 0.118318;
    textureColor.rgb = sum;
    //计算卡通纹理
    float leftIntensity = texture2D(inputImageTexture, leftTextureCoordinate).r;
    float rightIntensity = texture2D(inputImageTexture, rightTextureCoordinate).r;
    float topIntensity = texture2D(inputImageTexture, topTextureCoordinate).r;
    float topLeftIntensity = texture2D(inputImageTexture, topLeftTextureCoordinate).r;
    float topRightIntensity = texture2D(inputImageTexture, topRightTextureCoordinate).r;
    float bottomIntensity = texture2D(inputImageTexture, bottomTextureCoordinate).r;
    float bottomLeftIntensity = texture2D(inputImageTexture, bottomLeftTextureCoordinate).r;
    float bottomRightIntensity = texture2D(inputImageTexture, bottomRightTextureCoordinate).r;
    float h = -topLeftIntensity - 2.0 * topIntensity - topRightIntensity + bottomLeftIntensity + 2.0 * bottomIntensity + bottomRightIntensity;
    float v = -bottomLeftIntensity - 2.0 * leftIntensity - topLeftIntensity + bottomRightIntensity + 2.0 * rightIntensity + topRightIntensity;
    float mag = length(vec2(h, v));
    vec3 posterizedImageColor = floor((textureColor.rgb * quantizationLevels) + 0.5) / quantizationLevels;
    float thresholdTest = 1.0 - step(threshold, mag);
    vec4 tempColor = vec4(posterizedImageColor * thresholdTest, textureColor.a);
    gl_FragColor = mix(finalColor, tempColor, amplitude);
}