precision highp float;
varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform int imageWidth;
uniform int imageHeight;
uniform float progress;

uniform float persp;// 透视程度，= 0.7
uniform float unzoom;// 深度，即缩放程度 = 0.3
uniform float reflection;// 反射强度，= 0.4
uniform float floating;// 悬浮程度，= 3.0

#define SWAP_DIRECTION

vec4 getFromColor(vec2 p)
{
    #ifdef SWAP_DIRECTION
    p.y = 1.0-p.y;
    #endif
    return texture2D(inputImageTexture, p);
}

vec4 getToColor(vec2 p)
{
    #ifdef SWAP_DIRECTION
    p.y = 1.0-p.y;
    #endif
    return texture2D(inputImageTexture2, p);
}

vec2 project (vec2 p) {
    return p * vec2(1.0, -1.2) + vec2(0.0, -floating/100.);
}

bool inBounds (vec2 p) {
    return all(lessThan(vec2(0.0), p)) && all(lessThan(p, vec2(1.0)));
}

vec4 bgColor (vec2 p, vec2 pfr, vec2 pto) {
    vec4 c = vec4(0.0, 0.0, 0.0, 1.0);
    pfr = project(pfr);
    // FIXME avoid branching might help perf!
    if (inBounds(pfr)) {
        c += mix(vec4(0.0), getFromColor(pfr), reflection * mix(1.0, 0.0, pfr.y));
    }
    pto = project(pto);
    if (inBounds(pto)) {
        c += mix(vec4(0.0), getToColor(pto), reflection * mix(1.0, 0.0, pto.y));
    }
    return c;
}


vec2 xskew (vec2 p, float persp, float center) {
    float x = mix(p.x, 1.0-p.x, center);
    return (
    (
    vec2(x, (p.y - 0.5*(1.0-persp) * x) / (1.0+(persp-1.0)*x))
    - vec2(0.5-distance(center, 0.5), 0.0)
    )
    * vec2(0.5 / distance(center, 0.5) * (center<0.5 ? 1.0 : -1.0), 1.0)
    + vec2(center<0.5 ? 0.0 : 1.0, 0.0)
    );
}

vec4 transition(vec2 op) {
    #ifdef SWAP_DIRECTION
    op.y = 1.0-op.y;
    #endif
    float uz = unzoom * 2.0*(0.5-distance(0.5, progress));
    vec2 p = -uz*0.5+(1.0+uz) * op;
    vec2 fromP = xskew(
    (p - vec2(progress, 0.0)) / vec2(1.0-progress, 1.0),
    1.0-mix(progress, 0.0, persp),
    0.0
    );
    vec2 toP = xskew(
    p / vec2(progress, 1.0),
    mix(pow(progress, 2.0), 1.0, persp),
    1.0
    );
    // FIXME avoid branching might help perf!
    if (inBounds(fromP)) {
        return getFromColor(fromP);
    }
    else if (inBounds(toP)) {
        return getToColor(toP);
    }
    return bgColor(op, fromP, toP);
}


void main(void)
{
    vec4 resultColor = transition(textureCoordinate);
    gl_FragColor=resultColor;
}

