precision mediump float;

uniform sampler2D inputImageTexture;

varying vec2 textureCoordinate;
uniform float progress;
uniform float angleFactor;

const float PI = 3.14159265358979323846;

//判断 curCorrdinate 在两点 coordiante1 和  coordiante2 的左侧还是右侧
//Tmp > 0 在左侧 Tmp = 0 在线上 Tmp < 0 在右侧
float temp(vec2 coordiante1, vec2 coordiante2, vec2 curCorrdinate){
    float temp = (coordiante1.y - coordiante2.y) * curCorrdinate.x + (coordiante2.x - coordiante1.x) * curCorrdinate.y + coordiante1.x * coordiante2.y - coordiante2.x * coordiante1.y;
    return temp;
}

//获取当前坐标位于两条垂直直线的哪个区域
int getCellIndex(float angle, vec2 curCorrdinate){
    //第x象限为 x
    int cellIndex = -1;
    if (angle == 0.0 || angle == PI * angleFactor){
        //防止 tan(angle) 为 0 或者无穷时出现异常
        if (angle == 0.0){
            if (curCorrdinate.x >= 0.0 && curCorrdinate.y >= 0.0){
                cellIndex = 1;
            } else if (curCorrdinate.x > 0.0 && curCorrdinate.y < 0.0){
                cellIndex = 2;
            } else if (curCorrdinate.x < 0.0 && curCorrdinate.y < 0.0){
                cellIndex = 3;
            } else {
                cellIndex = 4;
            }
        } else {
            if (curCorrdinate.x >= 0.0 && curCorrdinate.y >= 0.0){
                cellIndex = 1;
            } else if (curCorrdinate.x > 0.0 && curCorrdinate.y < 0.0){
                cellIndex = 4;
            } else if (curCorrdinate.x < 0.0 && curCorrdinate.y < 0.0){
                cellIndex = 3;
            } else {
                cellIndex = 2;
            }
        }

    } else {
        float k1 = tan(angle);
        float k2 = -1.0/k1;
        float temp1 = temp(vec2(1.0, k1), vec2(0.0, 0.0), curCorrdinate);
        float temp2 = temp(vec2(1.0, k2), vec2(0.0, 0.0), curCorrdinate);
        if (temp2 < 0.0 && temp1 > 0.0){
            cellIndex = 1;
        } else if (temp1 < 0.0 && temp2 < 0.0){
            cellIndex = 2;
        } else if (temp1 < 0.0 && temp2 > 0.0){
            cellIndex = 3;
        } else {
            cellIndex = 4;
        }
    }
    return cellIndex;
}

void main(){

    float angle = PI * angleFactor * progress;
    vec2 newCoordinate = textureCoordinate;
    // move space from the center to the vec2(0.0)
    newCoordinate -= vec2(0.5);

    int cellIndex = getCellIndex(angle, newCoordinate);
    float distince = sin(angle)/2.0;//变化的距离 1/2 是最大的平移距离
    if (cellIndex == 1){
        //右移
        newCoordinate.x -= distince;
    } else if (cellIndex == 2){
        //上移
        newCoordinate.y -= distince;
    } else if (cellIndex == 3){
        //左移
        newCoordinate.x += distince;
    } else {
        //下移
        newCoordinate.y += distince;
    }

    // move it back to the original place
    newCoordinate += vec2(0.5);
    vec4 nColor=texture2D(inputImageTexture, newCoordinate);
    gl_FragColor=nColor;
}
