precision mediump float;
uniform sampler2D inputImageTexture;
varying vec2 textureCoordinate;

const int MIRROR_NONE = 0;//无镜像
const int MIRROR_HOR = 1;//水平镜像
const int MIRROR_VER = 2;//垂直镜像
const int MIRROR_ALL = 3;//水平垂直镜像
uniform int mirror;//镜像方向


//高斯模糊需要设置的值
uniform float texelWidthOffset;
uniform float texelHeightOffset;

//缩放的大小
uniform float scaleValueX;
uniform float scaleValueY;

//x 轴移动的大小
uniform float transXValue;
//y 轴移动的大小
uniform float transYValue;


//旋转的的大小
uniform float rotate;//输入为角度
uniform vec2 rotateAnchor;//旋转的中心点
uniform float inputHeight;
uniform float inputWidth;

uniform vec4 maskColor;//蒙版的颜色
uniform float maskAlpha;//蒙版 alpha 值

uniform float radialBlurSize;//0-100

//获取径向模糊后的颜色
vec4 getRadialBlurColor(sampler2D inputImageTexture, vec2 newTextureCoordinate, float radialBlurSize);

//获取高斯模糊后的颜色
vec4 getGaussBlurColor(sampler2D inputImageTexture, vec2 newTextureCoordinate, float texelWidthOffset, float texelHeightOffset);

//z 轴旋转
vec2 rotate2dCenter(vec2 newTextureCoordinate, vec2 rotateAnchor, float inputWidth, float inputHeight, float radianAngle);

//缩放效果
vec2 scale(vec2 newTextureCoordinate, vec2 scaleCenter, float scaleValueX, float scaleValueY);

//获取混合蒙版后的颜色
vec4 mixMaskColor(vec4 maskColor, vec4 inputColor, float maskAlpha, bool changeAlpha);

vec4 getFragColorByCondition(vec2 newTextureCoordinate){
    if (texelWidthOffset == 0.0 && texelHeightOffset == 0.0){
        if (radialBlurSize == 0.0){
            return texture2D(inputImageTexture, newTextureCoordinate);
        } else {
            //径向模糊
            return getRadialBlurColor(inputImageTexture, newTextureCoordinate, radialBlurSize);
        }
    } else {
        //高斯模糊
        return getGaussBlurColor(inputImageTexture, newTextureCoordinate, texelWidthOffset, texelHeightOffset);
    }
}


void main()
{
    vec2 newTextureCoordinate = textureCoordinate;
    //先执行偏移
    if (transXValue != 0.0 || transYValue != 0.0){
        newTextureCoordinate-=vec2(transXValue, transYValue);
    }

    if (rotate != 0.0){
        newTextureCoordinate = rotate2dCenter(newTextureCoordinate, rotateAnchor, inputWidth, inputHeight, rotate);
    }
    if (scaleValueX != 1.0 || scaleValueY != 1.0){
        newTextureCoordinate = scale(newTextureCoordinate, vec2(0.5), scaleValueX, scaleValueY);
    }

    vec4 resultColor = vec4(0.0, 0.0, 0.0, 0.0);
    bool hasChangeReultColor = true;
    if (mirror == MIRROR_ALL || (newTextureCoordinate.y >= 0.0  && newTextureCoordinate.y <= 1.0 && newTextureCoordinate.x >= 0.0  && newTextureCoordinate.x <= 1.0)){
        resultColor = getFragColorByCondition(newTextureCoordinate);
    } else if (mirror == MIRROR_HOR && (newTextureCoordinate.y >= 0.0  && newTextureCoordinate.y <= 1.0)){
        resultColor = getFragColorByCondition(newTextureCoordinate);
    } else if (mirror == MIRROR_VER && (newTextureCoordinate.x >= 0.0  && newTextureCoordinate.x <= 1.0)){
        resultColor = getFragColorByCondition(newTextureCoordinate);
    } else {
        hasChangeReultColor = false;
    }

    gl_FragColor = mixMaskColor(maskColor, resultColor, maskAlpha, hasChangeReultColor);
}
