precision mediump float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;//顶层
uniform sampler2D bgImageTexture;//背景

uniform float topTrans;//上半部分移动的距离
uniform float bottomTrans;//下半部分移动的距离
uniform float allTransX;//整体平移
uniform float scale;//缩放的大小

vec2 scaleCenter(vec2 newTextureCoordinate, float scaleValue){
    newTextureCoordinate -= vec2(0.5, 0.5);
    newTextureCoordinate = mat2(scaleValue, 0.0, 0.0, scaleValue) * newTextureCoordinate;
    newTextureCoordinate += vec2(0.5, 0.5);
    return newTextureCoordinate;
}

void main(){
    if (scale == 1.0 && allTransX == 0.0){
        if (topTrans == 0.0 && bottomTrans==0.0){
            gl_FragColor = texture2D(inputImageTexture, textureCoordinate);
        } else {
            vec2 newCoordinate = textureCoordinate;
            vec4 nColor;
            float originalX = newCoordinate.x;
            if (textureCoordinate.y < 0.5){
                //上半部分移动
                newCoordinate.x -= topTrans;
            } else {
                //下半部分移动
                newCoordinate.x -= bottomTrans;
            }
            bool useTopTexture = true;//使用顶部的 texture 绘制
            if (newCoordinate.x < 0.0 || newCoordinate.x > 1.0){
                newCoordinate.x = originalX;
                nColor = texture2D(bgImageTexture, newCoordinate);
            } else {
                nColor = texture2D(inputImageTexture, newCoordinate);
            }
            gl_FragColor=nColor;
        }
    } else {
        vec2 newCoordinate = textureCoordinate;
        //进行缩放
        if (scale != 1.0){
            newCoordinate = scaleCenter(newCoordinate, scale);
        }
        //进行整体平移
        if (allTransX != 0.0){
            newCoordinate.x -= allTransX;
        }
        gl_FragColor = texture2D(inputImageTexture, newCoordinate);
    }

}
