precision mediump float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D blendImageTexture;//背景
uniform vec3 innerGlowColor;//前景需要混合的颜色


//x 轴移动的大小
uniform float transXValue;
//y 轴移动的大小
uniform float transYValue;
//旋转的的大小
uniform float rotate;


//高斯模糊需要设置的值
const lowp int GAUSSIAN_SAMPLES = 9;
vec2 blurCoordinates[GAUSSIAN_SAMPLES];
uniform float texelWidthOffset;
uniform float texelHeightOffset;

uniform bool showMask;//是否显示蒙版
uniform vec4 maskColor;//蒙版的颜色

vec2 rotate2dCenter(vec2 newTextureCoordinate, float angle){
    newTextureCoordinate -= vec2(0.5);
    // rotate the space
    newTextureCoordinate = mat2(cos(angle), -sin(angle), sin(angle), cos(angle)) * newTextureCoordinate;
    // move it back to the original place
    newTextureCoordinate += vec2(0.5);
    return newTextureCoordinate;
}

void main(){
    if (showMask){
        gl_FragColor = maskColor;
    } else {
        vec2 newTextureCoordinate = textureCoordinate;
        newTextureCoordinate-=vec2(transXValue, transYValue);
        if (rotate != 0.0){
            newTextureCoordinate = rotate2dCenter(newTextureCoordinate, radians(rotate));
        }

        if (newTextureCoordinate.x < 0.0){
            newTextureCoordinate.x = -newTextureCoordinate.x;
        } else if (newTextureCoordinate.x > 1.0){
            newTextureCoordinate.x = 1.0 -(newTextureCoordinate.x - 1.0);
        }

        lowp vec4 color1 = texture2D(inputImageTexture, newTextureCoordinate);
        //blend
        lowp vec4 blendColor = texture2D(blendImageTexture, newTextureCoordinate);
        lowp vec4 resultColor = mix(color1, vec4(innerGlowColor.rgb, blendColor.a), blendColor.a);

        if (newTextureCoordinate.y >= 0.0 && newTextureCoordinate.y <= 1.0){
            gl_FragColor = vec4(resultColor.rgb, 1.0);
        } else {
            gl_FragColor = vec4(0.);
        }
    }
}