precision mediump float;
uniform sampler2D inputImageTexture;
varying vec2 textureCoordinate;

const lowp int GAUSSIAN_SAMPLES = 9;
vec2 blurCoordinates[GAUSSIAN_SAMPLES];
//高斯模糊需要设置的值
uniform float texelWidthOffset;
uniform float texelHeightOffset;

//获取高斯模糊后的颜色
vec4 getGaussBlurColor(vec2 newTextureCoordinate){
    lowp vec3 sum = vec3(0.0);
    // Calculate the positions for the blur
    int multiplier = 0;
    vec2 blurStep;
    vec2 singleStepOffset = vec2(texelHeightOffset, texelWidthOffset);

    for (int i = 0; i < GAUSSIAN_SAMPLES; i++)
    {
        multiplier = (i - ((GAUSSIAN_SAMPLES - 1) / 2));
        // Blur in x (horizontal)
        blurStep = float(multiplier) * singleStepOffset;
        blurCoordinates[i] = newTextureCoordinate.xy + blurStep;
    }

    lowp vec4 fragColor=texture2D(inputImageTexture, newTextureCoordinate);

    sum += texture2D(inputImageTexture, blurCoordinates[0]).rgb * 0.05;
    sum += texture2D(inputImageTexture, blurCoordinates[1]).rgb * 0.09;
    sum += texture2D(inputImageTexture, blurCoordinates[2]).rgb * 0.12;
    sum += texture2D(inputImageTexture, blurCoordinates[3]).rgb * 0.15;
    sum += texture2D(inputImageTexture, blurCoordinates[4]).rgb * 0.18;
    sum += texture2D(inputImageTexture, blurCoordinates[5]).rgb * 0.15;
    sum += texture2D(inputImageTexture, blurCoordinates[6]).rgb * 0.12;
    sum += texture2D(inputImageTexture, blurCoordinates[7]).rgb * 0.09;
    sum += texture2D(inputImageTexture, blurCoordinates[8]).rgb * 0.05;
    return vec4(sum, fragColor.a);
}

void main()
{
    vec2 newTextureCoordinate = textureCoordinate;
    if (texelWidthOffset == 0.0 && texelHeightOffset == 0.0){
        gl_FragColor = texture2D(inputImageTexture, newTextureCoordinate);
    } else {
        gl_FragColor = getGaussBlurColor(newTextureCoordinate);
    }


}