precision mediump float;

uniform sampler2D inputImageTexture;
uniform float angleFactor;

varying vec2 textureCoordinate;

const float limit1 = 1.0/3.0;
const float limit2 = 2.0/3.0;

const float PI = 3.14159265358979323846;

mat2 rotate2d(float angle){
    return mat2(cos(angle), -sin(angle), sin(angle), cos(angle));
}

//获取经过中心旋转后的坐标
vec2 getRotateCenterCooordinate(vec2 coordinate, float rotateAngle){
    vec2 newCoordinate = textureCoordinate;
    // move space from the center to the vec2(0.0)
    newCoordinate -= vec2(0.5);
    // rotate the space
    newCoordinate = rotate2d(rotateAngle) * newCoordinate;
    // move it back to the original place
    newCoordinate += vec2(0.5);
    return newCoordinate;
}

void main(){
    vec2 newCoordinate;
    if ((textureCoordinate.x <= limit1 && textureCoordinate.y <= limit1)
    || (textureCoordinate.x >= limit2 && textureCoordinate.y <= limit1)
    || (textureCoordinate.x > limit2 && textureCoordinate.y > limit2)
    || (textureCoordinate.x < limit1 && textureCoordinate.y > limit2)){
        //顺时针旋转
        newCoordinate = getRotateCenterCooordinate(textureCoordinate, angleFactor * PI);
    } else if (textureCoordinate.x > limit1 && textureCoordinate.x < limit2
    &&textureCoordinate.y > limit1 && textureCoordinate.y < limit2){
        //中心块不旋转
        newCoordinate = textureCoordinate;
    } else {
        //逆时针旋转
        newCoordinate = getRotateCenterCooordinate(textureCoordinate, -angleFactor * PI);
    }
    vec4 nColor=texture2D(inputImageTexture, newCoordinate);
    gl_FragColor=nColor;
}
