precision mediump float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform int step;

const float PI = 3.14159265358979323846;
const float halfBorder = 0.01;
const float range = 1.0/3.0 - halfBorder * 2.0;

float getLeft(float x) {
    if (x < 1.0/3.0) {
        return halfBorder;
    } else if (x < 2.0/3.0) {
        return 1.0/3.0 + halfBorder;
    } else if (x < 1.0) {
        return 2.0/3.0 + halfBorder;
    }
    return 0.0;
}

mat2 rotate2d(float angle){
    return mat2(cos(angle), -sin(angle), sin(angle), cos(angle));
}

void main(){

    if (step == 1) {
        if (textureCoordinate.x > 1.0/3.0 && textureCoordinate.x < 2.0/3.0 && textureCoordinate.y > 1.0/3.0 && textureCoordinate.y < 2.0/3.0) {
            gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
            return;
        }
    }

    if (step == 2) {
        if (!(textureCoordinate.x > 1.0/3.0 && textureCoordinate.x < 2.0/3.0 && textureCoordinate.y > 1.0/3.0 && textureCoordinate.y < 2.0/3.0)) {
            gl_FragColor = vec4(0.0);
            return;
        }
    }

    vec2 newCoordinate = textureCoordinate;
    newCoordinate = vec2((newCoordinate.x - getLeft(newCoordinate.x))/range, (newCoordinate.y - getLeft(newCoordinate.y))/range);

    if (newCoordinate.x < 0.0 || newCoordinate.x > 1.0 || newCoordinate.y < 0.0 || newCoordinate.y > 1.0) {
        gl_FragColor = vec4(0.0);
        return;
    }

    gl_FragColor = texture2D(inputImageTexture, newCoordinate);
}