precision highp float;
uniform sampler2D inputImageTexture;
varying vec2 textureCoordinate;

//旋转模糊需要设置的值
uniform float inputHeight;
uniform float inputWidth;
uniform float rotateBlurIntensity;

//获取旋转模糊后的颜色
vec4 getRotateBlurColor(sampler2D tex, vec2 center, vec2 resolution, vec2 curCoord, float intensity){
    vec2 dxy = curCoord - center;
    float r = length(dxy);
    float angle = atan(dxy.y, dxy.x);
    int num = 15;

    vec3 color = vec3(0.0);
    float step = 0.01;
    for (int i = 0; i < num; i++)
    {
        angle += (step * intensity);

        float  newX = r*cos(angle) + center.x;
        float  newY = r*sin(angle) + center.y;
        newX = abs(newX);
        if (newX > resolution.x)
        newX = resolution.x - mod(newX, resolution.x);
        newY = abs(newY);
        if (newY > resolution.y)
        newY = resolution.y - mod(newY, resolution.y);
        color += texture2D(tex, vec2(newX, newY)/resolution).rgb;
    }
    color /= float(num);
    return vec4(color, 1.0);
}


void main() {
    if (inputHeight > 0.0 && inputWidth > 0.0 && rotateBlurIntensity != 0.0){
        vec2 resolution = vec2(inputWidth, inputHeight);
        vec2 rotateCenter = resolution * 0.5;
        vec2 realCoord = textureCoordinate * resolution;
        gl_FragColor = getRotateBlurColor(inputImageTexture, rotateCenter, resolution, realCoord, rotateBlurIntensity);
    } else {
        gl_FragColor = texture2D(inputImageTexture, textureCoordinate);
    }
}