precision highp float;
uniform sampler2D inputImageTexture;
varying vec2 textureCoordinate;

const lowp int GAUSSIAN_SAMPLES = 9;

vec2 blurCoordinates[GAUSSIAN_SAMPLES];
uniform bool isMirror;//是否镜像
//高斯模糊需要设置的值
uniform float texelWidthOffset;
uniform float texelHeightOffset;
//缩放的大小
uniform float scaleValue;
uniform float scaleAnchorX;//缩放中心点 x
uniform float scaleAnchorY;//缩放中心点 y
//x 轴移动的大小
uniform float transXValue;
//y 轴移动的大小
uniform float transYValue;
//旋转的的大小
uniform float rotate;

//旋转模糊需要设置的值
uniform float inputHeight;
uniform float inputWidth;
uniform float rotateBlurIntensity;

vec2 scale(vec2 newTextureCoordinate, float scaleValue){
    newTextureCoordinate -= vec2(scaleAnchorX, scaleAnchorY);
    newTextureCoordinate = mat2(scaleValue, 0.0, 0.0, scaleValue) * newTextureCoordinate;
    newTextureCoordinate += vec2(scaleAnchorX, scaleAnchorY);
    return newTextureCoordinate;
}

vec2 rotate2dCenter(vec2 newTextureCoordinate, float angle){
    newTextureCoordinate -= vec2(0.5);
    // rotate the space
    newTextureCoordinate = mat2(cos(angle), -sin(angle), sin(angle), cos(angle)) * newTextureCoordinate;
    // move it back to the original place
    newTextureCoordinate += vec2(0.5);
    return newTextureCoordinate;
}

//获取旋转模糊后的颜色
vec4 getRotateBlurColor(sampler2D tex, vec2 center, vec2 resolution, vec2 curCoord, float intensity){
    vec2 dxy = curCoord - center;
    float r = length(dxy);
    float angle = atan(dxy.y, dxy.x);
    int num = 15;

    vec3 color = vec3(0.0);
    float step = 0.01;
    for (int i = 0; i < num; i++)
    {
        angle += (step * intensity);

        float  newX = r*cos(angle) + center.x;
        float  newY = r*sin(angle) + center.y;
        newX = abs(newX);
        if (newX > resolution.x)
        newX = resolution.x - mod(newX, resolution.x);
        newY = abs(newY);
        if (newY > resolution.y)
        newY = resolution.y - mod(newY, resolution.y);
        color += texture2D(tex, vec2(newX, newY)/resolution).rgb;
    }
    color /= float(num);
    return vec4(color, 1.0);
}


//获取高斯模糊后的颜色
vec4 getGaussBlurColor(vec2 newTextureCoordinate){
    lowp vec3 sum = vec3(0.0);
    // Calculate the positions for the blur
    int multiplier = 0;
    vec2 blurStep;
    vec2 singleStepOffset = vec2(texelHeightOffset, texelWidthOffset);

    for (int i = 0; i < GAUSSIAN_SAMPLES; i++)
    {
        multiplier = (i - ((GAUSSIAN_SAMPLES - 1) / 2));
        // Blur in x (horizontal)
        blurStep = float(multiplier) * singleStepOffset;
        blurCoordinates[i] = newTextureCoordinate.xy + blurStep;
    }

    lowp vec4 fragColor=texture2D(inputImageTexture, newTextureCoordinate);

    sum += texture2D(inputImageTexture, blurCoordinates[0]).rgb * 0.05;
    sum += texture2D(inputImageTexture, blurCoordinates[1]).rgb * 0.09;
    sum += texture2D(inputImageTexture, blurCoordinates[2]).rgb * 0.12;
    sum += texture2D(inputImageTexture, blurCoordinates[3]).rgb * 0.15;
    sum += texture2D(inputImageTexture, blurCoordinates[4]).rgb * 0.18;
    sum += texture2D(inputImageTexture, blurCoordinates[5]).rgb * 0.15;
    sum += texture2D(inputImageTexture, blurCoordinates[6]).rgb * 0.12;
    sum += texture2D(inputImageTexture, blurCoordinates[7]).rgb * 0.09;
    sum += texture2D(inputImageTexture, blurCoordinates[8]).rgb * 0.05;
    return vec4(sum, fragColor.a);
}

void main()
{
    vec2 newTextureCoordinate = textureCoordinate;
    if (rotate != 0.0){
        newTextureCoordinate = rotate2dCenter(newTextureCoordinate, rotate);
    }
    if (scaleValue != 1.0){
        newTextureCoordinate = scale(newTextureCoordinate, scaleValue);
    }
    newTextureCoordinate-=vec2(transXValue, transYValue);
    if (isMirror || (newTextureCoordinate.y >= 0.0  && newTextureCoordinate.y <= 1.0 && newTextureCoordinate.y <= 1.0 && newTextureCoordinate.x >= 0.0  && newTextureCoordinate.x <= 1.0)){
        if (texelWidthOffset == 0.0 && texelHeightOffset == 0.0){
            if (inputHeight > 0.0 && inputWidth > 0.0 && rotateBlurIntensity != 0.0){
                vec2 resolution = vec2(inputWidth, inputHeight);
                vec2 rotateCenter = resolution * 0.5;
                vec2 realCoord = newTextureCoordinate * resolution;
                gl_FragColor = getRotateBlurColor(inputImageTexture, rotateCenter, resolution, realCoord, rotateBlurIntensity);
            } else {
                gl_FragColor = texture2D(inputImageTexture, newTextureCoordinate);
            }
        } else {
            gl_FragColor = getGaussBlurColor(newTextureCoordinate);
        }
    } else {
        gl_FragColor = vec4(0.);
    }


}