precision mediump float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

//径向模糊
uniform float textureStepWidth;
uniform float textureStepHeight;
const vec2 radialBlurCenter = vec2(0.5, 0.5);
uniform float radialBlur;
uniform bool useRadialBlur;//是否有径向模糊

//灵魂出窍
uniform float spiriFreed;
float maxAlpha = 1.0;//透明度上限值
uniform float maxSpiriFreedScale;//图片放大的上限
uniform bool useSpiriFreed;//是否有灵魂出窍


uniform int showIndex;//显示的区域 0:全显示 1：左上 2：右上  3：右下 4：左下

uniform vec4 borderColor;//边框颜色
uniform float borderWidth;//边框宽度

uniform bool showMask;//是否显示蒙版
uniform vec4 maskColor;//蒙版的颜色


vec4 getValidColor(vec2 newTextureCoordinate, float weight){
    vec4 fragmentColor;
    if (newTextureCoordinate.x >= 0.0 && newTextureCoordinate.x <= 1.0 && newTextureCoordinate.y >= 0.0 && newTextureCoordinate.y <= 1.0){
        return texture2D(inputImageTexture, newTextureCoordinate) * weight;
    }
    return texture2D(inputImageTexture, textureCoordinate) * weight;
}

//获取径向模糊后的颜色
vec4 getRadialBlurColor(vec2 newTextureCoordinate){
    highp vec2 directionOfCoor = radialBlurCenter - newTextureCoordinate;
    directionOfCoor = directionOfCoor / length(directionOfCoor);
    highp vec2 samplingOffset = directionOfCoor * vec2(textureStepWidth, textureStepHeight) * radialBlur;

    highp float progress = 1.0;
    highp float tempProgress = 1.0 - progress;
    highp float weight1 = 0.227027;
    weight1 = weight1 + (1.0-weight1) * tempProgress;
    tempProgress = 1.0 - tempProgress;
    highp float weight2 = 0.1945946 * tempProgress;
    highp float weight3 = 0.1216216 * tempProgress;
    highp float weight4 = 0.054054 * tempProgress;
    highp float weight5 = 0.016216 * tempProgress;
    vec4 fragmentColor;

    fragmentColor = getValidColor(textureCoordinate, weight1);
    fragmentColor += getValidColor(textureCoordinate + samplingOffset, weight2);
    fragmentColor += getValidColor(textureCoordinate + (2.0 * samplingOffset), weight3);
    fragmentColor += getValidColor(textureCoordinate + (3.0 * samplingOffset), weight4);
    fragmentColor += getValidColor(textureCoordinate + (4.0 * samplingOffset), weight5);
    fragmentColor += getValidColor((textureCoordinate - samplingOffset), weight2);
    fragmentColor += getValidColor(textureCoordinate - (2.0 * samplingOffset), weight3);
    fragmentColor += getValidColor(textureCoordinate - (3.0 * samplingOffset), weight4);
    fragmentColor += getValidColor(textureCoordinate - (4.0 * samplingOffset), weight5);
    return fragmentColor;
}

//获取灵魂出窍后的颜色
vec4 getSpiriFreedColor(vec2 newTextureCoordinate){
    float alpha = maxAlpha * (1.0 - spiriFreed);//当前透明度[0.4，0]
    float scale = 1.0 + (maxSpiriFreedScale - 1.0) * spiriFreed;//当前缩放比例[1.0，1.8]
    //获取放大后的纹理坐标，将顶点坐标对应的纹理坐标的x/y值到中心点的距离，缩小一定的比例，仅仅只是改变了纹理坐标，而保持顶点坐标不变，从而达到拉伸效果
    //理解：纹理坐标映射到顶点坐标，中心点压缩后的坐标对应顶点坐标
    float weakX = 0.5 + (textureCoordinate.x - 0.5) / scale;
    float weakY = 0.5 + (textureCoordinate.y - 0.5) / scale;
    vec2 weakTextureCoords = vec2(weakX, weakY);
    vec4 weakMask = texture2D(inputImageTexture, weakTextureCoords);//获取当前像素点放大后的纹理坐标
    vec4 mask = texture2D(inputImageTexture, textureCoordinate);
    //颜色混合 内建函数mix / 混合方程式
    return mask * (1.0 - alpha) + weakMask * alpha;
}


void main(){
    vec4 resultColor = vec4(0.0, 0.0, 0.0, 0.0);
    vec2 newTextureCoordinate = textureCoordinate;
    if(showMask){
        gl_FragColor = maskColor;
    }else{
        if (useRadialBlur){
            resultColor = getRadialBlurColor(newTextureCoordinate);
        } else if (useSpiriFreed){
            resultColor = getSpiriFreedColor(newTextureCoordinate);
        } else {
            resultColor = texture2D(inputImageTexture, newTextureCoordinate);
        }
        gl_FragColor = resultColor;
    }
}